% [test_labels test_values C] = adaboost(test_data,train_data,...
%                                        train_labels,iterations);
%
% Run Adaboost classifier on decision stumps
% Learns on train set, predicts test set
%   train_data, test_data: #instances by #features data matrices
%   train_labels:          #instances vector (with values 1 or 2)
%   iterations:            integer array
%
% Author: Jean Honorio
% E-mail: jhonorio@csail.mit.edu
function [test_labels test_values C] = adaboost(test_data,train_data,...
                                                train_labels,iterations);

% Check input
[num_instances num_features] = size(train_data);
if size(test_data,2) ~= num_features
    error('Training and testing set have different number of features');
end
if length(train_labels) ~= num_instances
    error('Number of labels is different from number of instances in training set');
end

% Some pre-processing to make things faster
sorted_data = zeros(num_instances,num_features);
ind_data = zeros(num_instances,num_features,'int32');
sorted_sign = zeros(num_instances,num_features);
for f = 1:num_features

    % Sort training data for feature across instances
    [sorted_data_f ind_data_f] = sort(train_data(:,f));
    sorted_data(:,f) = sorted_data_f;
    ind_data(:,f) = ind_data_f;
    sorted_sign(:,f) = (train_labels(ind_data_f) == 1) - ...
                       (train_labels(ind_data_f) == 2);
end

% Assign initial weight to each sample
train_labels1 = find(train_labels == 1);
train_labels2 = find(train_labels == 2);
num1 = length(train_labels1);
num2 = num_instances - num1;
w = zeros(num_instances,1);
sum_w = 0;
for s = 1:num_instances
    if train_labels(s) == 1
        w(s) = 1 / num1;
    else
        w(s) = 1 / num2;
    end
    sum_w = sum_w + w(s);
end
w = w / sum_w;

% Iterate
smallest = 1e-4;
for t = 1:max(iterations)

    % Pick best classifier
    sum_w1 = sum(w(train_labels1));
    sum_w2 = sum(w(train_labels2));
    min_epsilon = inf;
    C.feature(t) = 0;
    C.theta(t) = 0;
    C.polarity(t) = 0;
    C_epsilon(t) = 0;
    for f = 1:num_features

        % Compute error for every possible threshold
        sorted_sign_w = sorted_sign(:,f) .* w(ind_data(:,f));
        cs_sorted_sign_w = cumsum(sorted_sign_w);
        epsilon_neg = sum_w2 + cs_sorted_sign_w;
        epsilon_pos = sum_w1 - cs_sorted_sign_w;

        for s = 1:num_instances - 1
            if sorted_data(s,f) ~= sorted_data(s + 1,f) & ...
               min(epsilon_neg(s),epsilon_pos(s)) < min_epsilon
                if epsilon_neg(s) < epsilon_pos(s)
                    epsilon = epsilon_neg(s);
                    polarity = -1;
                else
                    epsilon = epsilon_pos(s);
                    polarity = 1;
                end
                min_epsilon = epsilon;
                C.feature(t) = f;
                C.theta(t) = (sorted_data(s,f) + sorted_data(s + 1,f)) / 2;
                C.polarity(t) = polarity;
                C_epsilon(t) = epsilon;
            end
        end
    end

    % Check for premature end
    if C_epsilon(t) < smallest | C_epsilon(t) > 0.5
        if t > 1
            C.feature(t) = [];
            C.theta(t) = [];
            C.polarity(t) = [];
            C_epsilon(t) = [];
        else
            C_epsilon(t) = smallest;
        end
        break;
    end
    
    % Update weights
    beta = (1 - C_epsilon(t)) / C_epsilon(t);
    sum_w = 0;
    for s = 1:num_instances
        label = 2 - double(C.polarity(t)*train_data(s,C.feature(t)) < ...
                           C.polarity(t)*C.theta(t));
        if label ~= train_labels(s)
            w(s) = w(s) * beta;
        end
        sum_w = sum_w + w(s);
    end
    w = w / sum_w;
end
C.alpha = log((1 - C_epsilon) ./ C_epsilon);

% Predict on test data
num_test_instances = size(test_data,1);
oo = ones(num_test_instances,1);
test_values = zeros(num_test_instances,length(iterations));
for i = 1:length(iterations)
    T = min(iterations(i),length(C.alpha));
    test_values(:,i) = sum((oo*C.alpha(1:T)).*((oo*C.polarity(1:T)).*test_data(:,C.feature(1:T)) < ...
                                               oo*(C.polarity(1:T).*C.theta(1:T))),2) - sum(C.alpha(1:T))/2;
end
test_labels = 2 - double(test_values > 0);
