% [data hdr] = load_contrasts(ff_contrasts,contrasts,...
%                             subjects,scaling);
%
% Load all subjects and contrasts
%   ff_contrasts: string containing [subject] and [contrast]
%                 e.g. 'c:\myfiles\con_[subject]_[contrast].img
%   contrasts:    #contrasts string list
%   subjects:     #subjects string list
%   scaling:      0 for none
%                 1 for grand mean scaling
%                 2 for standard deviation scaling 
%   data:         5-D array (#subjects x #contrasts x sizeX x sizeY x sizeZ)
%   hdr:          header structure in case you wanna save volumes
%
% Author: Jean Honorio
% E-mail: jhonorio@csail.mit.edu
function [data hdr] = load_contrasts(ff_contrasts,contrasts,...
                                     subjects,scaling);

% Load all subjects and contrasts
fprintf('Loading all subjects and contrasts...\n');
first = true;
for s = 1:length(subjects)
    for c = 1:length(contrasts)
        fprintf('Subject %s, contrast %s...\n',subjects{s},contrasts{c});
        file_name = ff_contrasts;
        file_name = strrep(file_name,'[subject]',subjects{s});
        file_name = strrep(file_name,'[contrast]',contrasts{c});
        if ~exist(file_name,'file')
            error('Contrast map does not exist');
        end
        hdr = spm_vol(file_name);
        vol = spm_read_vols(hdr);
        if strcmp(scaling,'none')
            % Nothing to do
        elseif strcmp(scaling,'mult100')
            % Structural MRI goes from 0 to 1, just to keep same scale as fMRI
            vol = vol * 100;
        elseif strcmp(scaling,'gmsca100')
            % Grand mean scaling to 100
            vol = vol * 100 / abs(spm_global(hdr));
        elseif strcmp(scaling,'stdev100')
            % Dangerous! there is no bibliography for this
            vol = vol * 100 / std(vol(~isnan(vol(:))));
        else
            error('Not a valid scaling');
        end
        if first
            [X Y Z] = size(vol);
            data = zeros(length(subjects),length(contrasts),X,Y,Z);
            first = false;
        end
        data(s,c,:,:,:) = vol;
    end
end
hdr.fname = 'cleared to avoid possible problems';
