% select_features(ff_contrasts,root_output,task,contrast,...
%                 subjects,labels,fold_subject,scaling,C);
%
% Select features for all leave-one-out folds (after voxel ranking)
%   subjects:  #subjects string list
%   labels:    #subjects vector (with values 1 or 2)
%   C:         structure as in "feature_name"
%
% Author: Jean Honorio
% E-mail: jhonorio@csail.mit.edu
function select_features(ff_contrasts,root_output,task,contrast,...
                         subjects,labels,fold_subject,scaling,C);

% Set method
warning off;
if strcmp(C.method,'cluster')
    type = 'cluster';
    rankings = 'stump';
elseif strcmp(C.method,'pca')
    type = 'pca';
    rankings = '';
elseif strcmp(C.method,'ica')
    type = 'ica';
    rankings = '';
elseif strcmp(C.method,'cubes')
    type = 'cubes';
    rankings = 'stump'; % only needs the 'mask.img' file
elseif strcmp(C.method,'discrim')
    type = 'top';
    rankings = 'normal';
elseif strcmp(C.method,'active')
    type = 'top';
    rankings = 'tstat';
elseif strcmp(C.method,'slight')
    type = 'top';
    rankings = 'gnb27';
elseif strcmp(C.method,'rfesvm')
    type = 'rfe';
    rankings = '';
    rfe_weights = @(train_data,train_labels) svmlin_weights(train_data,train_labels);
else
    error('Not a valid feature selection method');
end

% Check fold array
if isempty(fold_subject)
    fold_subject = 1:length(subjects);
end
check_fold_array(fold_subject);

% Create output directory if it doesn't exist
root_features = fullfile(root_output,['features_',feature_name(C)],task);
mkdir(root_features);

% Load all subjects, if needed
if strcmp(rankings,'')
    [data hdr] = load_contrasts(ff_contrasts,{contrast},subjects,scaling);
    valid = ~isnan(squeeze(sum(data)));
else
    root_rankings = fullfile(root_output,['rankings_',rankings],task);
    file_name = fullfile(root_rankings,'mask.img');
	fprintf('Loading %s...\n',file_name);
    hdr = spm_vol(file_name);
    valid = spm_read_vols(hdr);
end
hdr.fname = fullfile(root_features,'mask.img');
fprintf('Saving %s...\n',hdr.fname);
spm_write_vol(hdr,valid);

% Select features
[X Y Z] = size(valid);
if strcmp(type,'pca') | strcmp(type,'ica')

    % Go through each fold
    for s = 0:max(fold_subject)
        if s == 0
            fprintf('All subjects\n');
            who = 'all';
            idx_s = 1:length(subjects);
        else
            fprintf('Fold %d\n',s);
            idx_sf = find(fold_subject == s);
            fprintf('Subject');
            who = 'without';
            for sf = idx_sf
                fprintf(' %s',subjects{sf});
                who = [who '_' subjects{sf}];
            end
            fprintf('\n');
            idx_s = setdiff(1:length(subjects),idx_sf);
        end
        file_name = [task,'_',who];
        if length(file_name) > 150
            file_name = [file_name(1:145) '_more'];
        end

        % PCA and ICA
        data_flat = reshape(data(idx_s,:,:,:,:),length(idx_s),[]);
        valid_flat = reshape(valid,1,[]);
        data_flat = data_flat(:,valid_flat);
        [U S] = svd(remmean(data_flat.'),'econ');
        num_features_s = min(C.components,length(idx_s)-1);
        if strcmp(type,'pca')

            % PCA only
            weights = U(:,1:num_features_s);
        else

            % Whitening by PCA, then ICA
            E = U(:,1:num_features_s);
            D = S(1:num_features_s,1:num_features_s).^2/length(subjects);
            A = [];
            trials = 0;
            while (isempty(A) | size(A,2) < num_features_s) & trials < 20
                if trials >= 10
                    stab = 'on';
                else
                    stab = 'off';
                end
                if trials > 0
                    fprintf('Trying ICA %d times (stabilization=%s)\n',trials+1,stab);
                end
                [A tmp] = fastica(data_flat.','pcaE',E,'pcaD',D,'stabilization',stab,'verbose','off');
                trials = trials+1;
            end
            if isempty(A) | size(A,2) < num_features_s
                error('ICA didnt converge');
            end
            weights = A;
        end 

        % Save each component (feature) as one volume
        for f = 1:num_features_s
            vol = nan(X,Y,Z);
            vol(valid_flat) = weights(:,f);
            hdr.fname = fullfile(root_features,[file_name,'_',sprintf('%d',f),'.img']);
            fprintf('Saving %s...\n',hdr.fname);
            spm_write_vol(hdr,vol);
        end
    end
elseif strcmp(type,'cubes')
    
    % Generate cubes
    vol = nan(X,Y,Z);
    cube_size = floor([X Y Z] / C.size);
    cube_start = floor(([X Y Z] - cube_size * C.size) / 2) + 1;
    num_cubes = 0;
    for i = 1:cube_size(1)
        x1 = cube_start(1) + C.size * (i - 1);
        x2 = cube_start(1) + C.size * i - 1;
        for j = 1:cube_size(2)
            y1 = cube_start(2) + C.size * (j - 1);
            y2 = cube_start(2) + C.size * j - 1;
            for k = 1:cube_size(3)
                z1 = cube_start(3) + C.size * (k - 1);
                z2 = cube_start(3) + C.size * k - 1;
                
                % At least 10% of voxels must be valid
                if sum(sum(sum(valid(x1:x2,y1:y2,z1:z2)))) >= 0.1*C.size^3
                    num_cubes = num_cubes+1;
                    vol(x1:x2,y1:y2,z1:z2) = num_cubes;
                end
            end
        end
    end
    vol(~valid) = nan;
    
    % Save volume with labels (features)
    hdr.fname = fullfile(root_features,[task,'_all.img']);
    fprintf('Saving %s...\n',hdr.fname);
    spm_write_vol(hdr,vol);
elseif strcmp(type,'top')
    
    % Process every leave-one-out from voxel ranking results
    files1 = dir(fullfile(root_rankings,[task,'_all.img']));
    files2 = dir(fullfile(root_rankings,[task,'_without_*.img']));
    files = [files1(:); files2(:)];
    valid_list = find(valid(:));
    for i = 1:length(files)

        % Load volume
        file_name = fullfile(root_rankings,files(i).name);
        fprintf('Loading %s...\n',file_name);
        hdr = spm_vol(file_name);
        vol = spm_read_vols(hdr);
        
        % Get voxels with highest values
        valid_vol = vol(valid_list);
        valid_vol(isnan(valid_vol)) = -inf;
        [tmp highest] = sort(valid_vol,'descend');
        vol = nan(X,Y,Z);
        for f = 1:C.voxels
            vol(valid_list(highest(f))) = f;
        end

        % Save volume with labels (features)
        hdr.fname = fullfile(root_features,files(i).name);
        fprintf('Saving %s...\n',hdr.fname);
        spm_write_vol(hdr,vol);
    end
elseif strcmp(type,'cluster')
    
    % Process every leave-one-out from voxel ranking results
    files1 = dir(fullfile(root_rankings,[task,'_all.img']));
    files2 = dir(fullfile(root_rankings,[task,'_without_*.img']));
    files = [files1(:); files2(:)];
    valid_list = find(valid(:));
    for i = 1:length(files)

        % Load volume
        file_name = fullfile(root_rankings,files(i).name);
		fprintf('Loading %s...\n',file_name);
        hdr = spm_vol(file_name);
        vol = spm_read_vols(hdr);
        
        % Remove voxels if not on top percentile
        v = quantile(vol(valid_list), C.quantile/100);
        vol(vol < v) = nan;

        % Get all clusters
		coords = [];
		for z = 1:Z
			[x y] = find(~isnan(vol(:,:,z)));
			if length(x) > 0
				coords = [coords; x y z*ones(length(x),1)];
			end
		end
        clusters = spm_clusters(coords.');
        
        % Get biggest cluster
        num_clusters = max(clusters);
        cluster_size = zeros(1,num_clusters);
        for c = 1:num_clusters
            idx = find(clusters == c);
            cluster_size(c) = length(idx);
        end
		[tmp biggest] = max(cluster_size);
		vol = nan(X,Y,Z);
		idx = find(clusters == biggest);
		for j = idx
			vol(coords(j,1),coords(j,2),coords(j,3)) = 1;
		end

        % Save volume with labels (one feature only)
        hdr.fname = fullfile(root_features,files(i).name);
        fprintf('Saving %s...\n',hdr.fname);
        spm_write_vol(hdr,vol);
    end
elseif strcmp(type,'rfe')

    % Go through each fold
    valid_list = find(valid(:));
    for s = 0:max(fold_subject)
        if s == 0
            fprintf('All subjects\n');
            who = 'all';
            idx_s = 1:length(subjects);
        else
            fprintf('Fold %d\n',s);
            idx_sf = find(fold_subject == s);
            fprintf('Subject');
            who = 'without';
            for sf = idx_sf
                fprintf(' %s',subjects{sf});
                who = [who '_' subjects{sf}];
            end
            fprintf('\n');
            idx_s = setdiff(1:length(subjects),idx_sf);
        end
        file_name = [task,'_',who];
        if length(file_name) > 150
            file_name = [file_name(1:145) '_more'];
        end

        % Recursive feature elimination with SVM
        data_flat = reshape(data(idx_s,:,:,:,:),length(idx_s),[]);
        valid_flat = reshape(valid,1,[]);
        data_flat = data_flat(:,valid_flat);
        highest = 1:size(data_flat,2);
        while length(highest) > C.voxels
            weight = rfe_weights(data_flat(:,highest),labels(idx_s));
            [tmp idx] = sort(abs(weight));
            remove = C.remove;
            if length(highest) - remove < C.voxels
                remove = length(highest) - C.voxels;
            end
            highest(idx(1:remove)) = [];
        end

        % Save volume with labels (features)
        vol = nan(X,Y,Z);
        for f = 1:C.voxels
            vol(valid_list(highest(f))) = f;
        end
        hdr.fname = fullfile(root_features,[file_name,'.img']);
        fprintf('Saving %s...\n',hdr.fname);
        spm_write_vol(hdr,vol);
    end
else
    error('This should not happen');
end

% Weights of an SVM classifier
function w = svmlin_weights(train_data,train_labels);

K.kernel = 'linear';
[tmp tmp C] = svm(train_data,train_data,train_labels,K);
w = C.weight(1:end-1);
