% [test_labels test_values C] = svm(test_data,train_data,train_labels,K);
%
% Run linear Support Vector Machine classifier
% Learns on train set, predicts test set
%   train_data, test_data: #instances by #features data matrices
%   train_labels:          #instances vector (with values 1 or 2)
%   K:                     structure as follows
% Linear kernel:   K.kernel = 'linear'
% Gaussian kernel: K.kernel = 'rbf', K.gamma = double
%
% Author: Jean Honorio
% E-mail: jhonorio@csail.mit.edu
function [test_labels test_values C] = svm(test_data,train_data,train_labels,K);

% Check input
[num_instances num_features] = size(train_data);
if size(test_data,2) ~= num_features
    error('Training and testing set have different number of features');
end
if length(train_labels) ~= num_instances
    error('Number of labels is different from number of instances in training set');
end

% Learn on training data
options = optimset('Display','off');
if strcmp(K.kernel,'linear')
    % Nothing to do
elseif strcmp(K.kernel,'rbf')
    train_data = train_data / K.gamma;
    test_data = test_data / K.gamma;
else
    error('Not a valid kernel');
end
h = svmtrain(train_data,train_labels,'kernel_function',K.kernel,'quadprog_opts',options);
C.weight = [h.SupportVectors.'*h.Alpha; h.Bias];

% Predict on test data
test_values = [test_data, ones(size(test_data,1),1)] * C.weight;
try
    % SVMCLASSIFY fails when result is exactly zero
    % SVMDECISION uses sign
    test_labels = svmclassify(h,test_data);
    if size(test_data,1)~=length(test_labels)
        error('unexpected');
    end
catch
    if strcmp(K.kernel,'linear')
        warning('svm: svmclassify failed\n');
        test_labels = 2 - double(test_values > 0);
    else
        error('svm: svmclassify failed\n');
    end
end
